/*******************************************************************************
Copyright © 2023 Sebastian Roith

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*******************************************************************************/

(function(window) {

const LEVELS = [
    {
        // Angezeigter Arbeitsauftrag (HTML erlaubt)
        task: 'Was dieser Algorithmus bewirkt, sieht man ihm nun wirklich nicht auf den ersten Blick an. Löse das Rätsel, indem du ihn dem Roboter ausführen lässt.',

        // Welt und Roboterstartposition als Maschinencode
        world: '0166000000000000000000000000000000000000000000000000000000000000000000000000000',

        // Quelltext, der anfangs im Editor steht
        code: 'plus\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nlinksrum\nlinksrum\nlinksrum\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nlinksrum\nlinksrum\nlinksrum\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nlinksrum\nlinksrum\nlinksrum\nvor\nplus\nvor\nplus\nvor\nplus\nlinksrum\nlinksrum\nlinksrum\nvor\nplus\nvor\nplus\nlinksrum\nlinksrum\nlinksrum\nvor\nplus',

        // Ist das Bearbeiten des Quelltextes gesperrt? (Optional, Standardwert false)
        readonly: true,

        // Muss - ggf. zusätzlich zur korrekten Fragenbeantwortung - checkCode oder checkTermination unlock=true liefern, bevor Level abgeschlossen werden kann? (Optional, Standardwert false)
        locked: true,

        // Optional: Reagiert auf ein Ende der Algorithmusausführung; Rückgabewert wie checkCode
        checkTermination: [ CodeCheckers.accept(true) ],

        // Wird falsches Beantworten der Frage als Fehlversuch gewertet? (Optional, Standardwert false)
        questionWrongFine: 0,

        // Falls erst eine Frage richtig beantwortet werden muss: Welche?
        question: null
    },
    {
        task: 'Damit es etwas übersichtlicher wird: Bringe dem Roboter die Anweisung <code>rechtsrum</code> bei und ersetze alle drei aufeinander folgenden Aufrufe von <code>linksrum</code> durch <code>rechtsrum</code>:',
        world: '0366000000000000000000000000000000000000000000000000000000000000000000000000000',
        code: 'plus\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nlinksrum\nlinksrum\nlinksrum\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nlinksrum\nlinksrum\nlinksrum\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nlinksrum\nlinksrum\nlinksrum\nvor\nplus\nvor\nplus\nvor\nplus\nlinksrum\nlinksrum\nlinksrum\nvor\nplus\nvor\nplus\nlinksrum\nlinksrum\nlinksrum\nvor\nplus',
        keepCode: true,
        locked: true,
        checkCode: [
            CodeCheckers.allowOnlyCommandsAndDefs(0),
            CodeCheckers.rejectMissingUserDefined(1, 'rechtsrum'),
            CodeCheckers.rejectAdditionalUserDefined(0, ['rechtsrum']),
            CodeCheckers.accept()
        ],
        checkTermination: [
            CodeCheckers.ensureUserDefinedAdd(1, 'rechtsrum', 'linksrum\nlinksrum\nlinksrum'),
            CodeCheckers.ensureMain(1, 'plus\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nrechtsrum\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nrechtsrum\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nrechtsrum\nvor\nplus\nvor\nplus\nvor\nplus\nrechtsrum\nvor\nplus\nvor\nplus\nrechtsrum\nvor\nplus'),
            CodeCheckers.acceptUserAbort(),
            CodeCheckers.accept(true)
        ]
    },
    {
        task: 'Es wird ja auch ständig erst <code>vor</code> und dann <code>plus</code> ausgeführt. Das soll künftig die selbstdefinierte Anweisung <code>gerade</code> erledigen. Definiere diese Anweisung und nutze sie, um den Algorithmus noch ein Stück übersichtlicher zu schreiben.',
        world: '0366000000000000000000000000000000000000000000000000000000000000000000000000000',
        code: '# Insgesamt um 90° nach links drehen\nLERNE rechtsrum\n    linksrum\n    linksrum\n    linksrum\n\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nrechtsrum\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nrechtsrum\nvor\nplus\nvor\nplus\nvor\nplus\nvor\nplus\nrechtsrum\nvor\nplus\nvor\nplus\nvor\nplus\nrechtsrum\nvor\nplus\nvor\nplus\nrechtsrum\nvor\nplus',
        keepCode: true,
        locked: true,
        checkCode: [
            CodeCheckers.allowOnlyCommandsAndDefs(0),
            CodeCheckers.rejectMissingUserDefined(1, 'rechtsrum'),
            CodeCheckers.rejectMissingUserDefined(1, 'gerade'),
            CodeCheckers.rejectAdditionalUserDefined(0, ['rechtsrum', 'gerade']),
            CodeCheckers.accept()
        ],
        checkTermination: [
            CodeCheckers.ensureUserDefinedUnchanged(1, 'rechtsrum', 'linksrum\nlinksrum\nlinksrum'),
            CodeCheckers.ensureUserDefinedAdd(1, 'gerade', 'vor\nplus'),
            CodeCheckers.ensureMain(1, 'plus\ngerade\ngerade\ngerade\ngerade\ngerade\nrechtsrum\ngerade\ngerade\ngerade\ngerade\ngerade\nrechtsrum\ngerade\ngerade\ngerade\ngerade\nrechtsrum\ngerade\ngerade\ngerade\nrechtsrum\ngerade\ngerade\nrechtsrum\ngerade'),
            CodeCheckers.acceptUserAbort(),
            CodeCheckers.accept(true)
        ]
    },
    {
        task: 'Das ist doch schon besser. Vereinfache den Algorithmus nun noch weiter durch eine selbstdefinierte Anweisung <code>ecke</code>. Diese soll dasselbe wie zweimal <code>gerade</code> bewirken, nur mit dem Unterschied, dass sich der Roboter dazwischen nach rechts dreht. Verwende in <code>ecke</code> die beiden anderen selbstdefinierten Anweisungen wo immer sinnvoll möglich.',
        world: '0366000000000000000000000000000000000000000000000000000000000000000000000000000',
        code: '# Insgesamt um 90° nach links drehen\nLERNE rechtsrum\n    linksrum\n    linksrum\n    linksrum\n\n# Auf nächstes Feld ein Plättchen legen\nLERNE gerade\n    vor\n    plus\n\nplus\ngerade\ngerade\ngerade\ngerade\ngerade\nrechtsrum\ngerade\ngerade\ngerade\ngerade\ngerade\nrechtsrum\ngerade\ngerade\ngerade\ngerade\nrechtsrum\ngerade\ngerade\ngerade\nrechtsrum\ngerade\ngerade\nrechtsrum\ngerade',
        keepCode: true,
        locked: true,
        checkCode: [
            CodeCheckers.allowOnlyCommandsAndDefs(0),
            CodeCheckers.rejectMissingUserDefined(1, 'rechtsrum'),
            CodeCheckers.rejectMissingUserDefined(1, 'gerade'),
            CodeCheckers.rejectMissingUserDefined(1, 'ecke'),
            CodeCheckers.rejectAdditionalUserDefined(0, ['rechtsrum', 'gerade', 'ecke']),
            CodeCheckers.accept()
        ],
        checkTermination: [
            CodeCheckers.ensureUserDefinedUnchanged(1, 'rechtsrum', 'linksrum\nlinksrum\nlinksrum'),
            CodeCheckers.ensureUserDefinedUnchanged(1, 'gerade', 'vor\nplus'),
            CodeCheckers.ensureUserDefinedAdd(1, 'ecke', 'gerade\nrechtsrum\ngerade'),
            CodeCheckers.ensureMain(1, 'plus\ngerade\ngerade\ngerade\ngerade\necke\ngerade\ngerade\ngerade\necke\ngerade\ngerade\necke\ngerade\necke\necke'),
            CodeCheckers.acceptUserAbort(),
            CodeCheckers.accept(true)
        ]
    },
    {
        task: 'Warum der Algorithmus besser ist, als der ursprüngliche? Ganz klar: Zum einen ist er kürzer und leichter verständlich. Zum anderen einfacher anpassbar: Jetzt steht der Roboter auf der anderen Seite seiner Welt. Er soll wieder eine Schneckenlinie legen. Diesmal soll er an jeder Ecke aber statt rechts links weitermachen. Passe den Algorithmus entsprechend an &dash; mit nur einer minimalen Änderung:',
        world: '0366000000000000000000000000000000000000000000000000000000000000000000000000500',
        code: '# Insgesamt um 90° nach links drehen\nLERNE rechtsrum\n    linksrum\n    linksrum\n    linksrum\n\n# Auf nächstes Feld ein Plättchen legen\nLERNE gerade\n    vor\n    plus\n\n# Zwei Felder über Ecke legen\nLERNE ecke\n    gerade\n    rechtsrum\n    gerade\n\nplus\ngerade\ngerade\ngerade\ngerade\necke\ngerade\ngerade\ngerade\necke\ngerade\ngerade\necke\ngerade\necke\necke',
        keepCode: true,
        locked: true,
        checkCode: [
            CodeCheckers.allowOnlyCommandsAndDefs(0),
            CodeCheckers.rejectMissingUserDefined(1, 'gerade'),
            CodeCheckers.rejectMissingUserDefined(1, 'ecke'),
            CodeCheckers.rejectAdditionalUserDefined(0, ['rechtsrum', 'gerade', 'ecke']),
            CodeCheckers.accept()
        ],
        checkTermination: [
            CodeCheckers.ensureUserDefinedUnchanged(1, 'gerade', 'vor\nplus'),
            CodeCheckers.ensureUserDefinedChanged(1, 'ecke', 'gerade\nrechtsrum\ngerade', 'gerade\nlinksrum\ngerade'),
            CodeCheckers.ensureMain(1, 'plus\ngerade\ngerade\ngerade\ngerade\necke\ngerade\ngerade\ngerade\necke\ngerade\ngerade\necke\ngerade\necke\necke'),
            CodeCheckers.acceptUserAbort(),
            CodeCheckers.accept(true)
        ]
    }
];

window.roboterCodingTask = {
    levels: LEVELS
};
})(window);
